"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const source_map_1 = require("source-map");
const convert_source_map_1 = require("convert-source-map");
const stringPos = require('string-pos');
function check(files) {
    return Promise.all(files.map((file) => __awaiter(this, void 0, void 0, function* () {
        if (!file.url.endsWith('.js')) {
            return {
                type: 'UNSUPPORTED_FILE',
                url: file.url
            };
        }
        const sourceMap = convert_source_map_1.fromSource(file.text);
        if (!sourceMap) {
            return {
                type: 'NO_SOURCE_MAP',
                url: file.url
            };
        }
        const results = yield source_map_1.SourceMapConsumer.with(sourceMap.toObject(), null, (consumer) => {
            return consumer.sources
                .map((source) => {
                const coverage = calculateUnusedCode(consumer, source, file);
                return { coverage, source };
            })
                .filter(({ coverage }) => !isNaN(coverage)) // TODO: Check, why this can be NaN.
                .sort((a, b) => a.coverage - b.coverage);
        });
        return { type: 'REPORT', url: file.url, results };
    })));
}
exports.check = check;
function calculateUnusedCode(consumer, source, file) {
    const generatedStart = consumer.generatedPositionFor({
        source,
        line: 1,
        column: 0
    });
    const content = consumer.sourceContentFor(source);
    const generatedEnd = consumer.generatedPositionFor(Object.assign({ source }, stringPos(content, content.length)));
    const startIndex = stringPos.toIndex(file.text, generatedStart);
    const endIndex = stringPos.toIndex(file.text, generatedEnd);
    const overlap = file.ranges.reduce((acc, range) => {
        const totalRange = Math.max(range.end, endIndex) - Math.min(range.start, startIndex);
        const sumOfRanges = range.end - range.start + (endIndex - startIndex);
        if (sumOfRanges > totalRange) {
            acc += Math.min(range.end, endIndex) - Math.max(range.start, startIndex);
        }
        return acc;
    }, 0);
    return overlap / (endIndex - startIndex);
}
//# sourceMappingURL=check.js.map