"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const commander_1 = __importDefault(require("commander"));
const check_1 = require("./check");
const { version } = require('../package.json');
commander_1.default
    .version(version)
    .option('-f, --file <path>', 'Path to .json file')
    .option('-t, --threshold <value>', 'Only report files below a certain threshold', parseFloat, 0.5)
    .action(({ file, threshold }) => {
    if (file) {
        showReport({ file, threshold }).catch((err) => {
            console.error(err.toString());
            process.exitCode = 1;
        });
    }
    else {
        commander_1.default.help();
    }
})
    .parse(process.argv);
function showReport({ file, threshold }) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = require(path_1.resolve(file));
        const fileReports = yield check_1.check(data);
        fileReports.forEach((fileReport) => {
            switch (fileReport.type) {
                case 'UNSUPPORTED_FILE':
                    console.log(`"${fileReport.url}" is no JS file. Skipped.`);
                    break;
                case 'NO_SOURCE_MAP':
                    console.log(`"${fileReport.url}" has no source map. Skipped.`);
                    break;
                case 'REPORT':
                    const { url } = fileReport;
                    const thresholdPercent = Math.round(threshold * 100);
                    const filtered = fileReport.results.filter(({ coverage }) => coverage < threshold);
                    if (filtered.length === 0) {
                        console.log(`"${url}" has no file with less than ${thresholdPercent}% unused code.`);
                    }
                    else if (filtered.length === 1) {
                        console.log(`"${url}" one file with less than ${thresholdPercent}% unused code:`);
                        const percent = Math.round(filtered[0].coverage * 100);
                        console.log(`  Unused code in "${filtered[0].source}": ${percent}%`);
                    }
                    else {
                        const count = filtered.length;
                        console.log(`"${url}" has ${count} files with less than ${thresholdPercent}% unused code:`);
                        filtered.forEach((result) => {
                            const percent = Math.round(result.coverage * 100);
                            console.log(`  Unused code in "${result.source}": ${percent}%`);
                        });
                    }
                    break;
            }
            console.log('');
        });
    });
}
//# sourceMappingURL=cli.js.map